package page_vote

import (
	"code.electrolab.fr/it/vote.electrolab.fr/application"
	"context"
	"database/sql"
	"html/template"
	"log"
	"math/rand"
	"net/http"
	"strings"
	"time"
)

const (
	FORM_VOTER_PRIVATE_TOKEN = "private_token"
)

var (
	templateChoice    = template.Must(template.New("index").Parse(TEMPLATE_CHOICE_STR))
	templateScore     = template.Must(template.New("index").Parse(TEMPLATE_SCORE_STR))
	templateForbidden = template.Must(template.New("forbidden").Parse(TEMPLATE_FORBIDDEN_STR))
)

func RegisterRoutes(mux *http.ServeMux, app *application.App) {
	mux.Handle("/votes/", application.NewAppHandler(app, handleGetVote))
}

func handleGetVote(app *application.App, ctx context.Context, w http.ResponseWriter, r *http.Request) {
	var (
		template     *template.Template
		templateVars = new(TemplateVars)
	)

	pathParts := strings.Split(r.URL.Path, "/")
	if len(pathParts) != 3 {
		w.WriteHeader(http.StatusNotFound)
		return
	}

	questionName := pathParts[2]
	err := app.DB.QueryRowContext(ctx, `SELECT name, title, position, sentence, min_choices, COALESCE(max_choices, -1), vote_open FROM questions WHERE name = $1 ORDER BY position`, questionName).Scan(&templateVars.Name, &templateVars.Title, &templateVars.Position, &templateVars.Sentence, &templateVars.MinChoices, &templateVars.MaxChoices, &templateVars.VoteOpen)
	if err != nil {
		if err == sql.ErrNoRows {
			// Question is not found
			// w.Header().Set("Location", "/votes")
			w.WriteHeader(http.StatusNotFound)
			return
		} else {
			// Query has an error
			log.Printf("Query error 1: %s", err.Error())
			w.WriteHeader(http.StatusInternalServerError)
			return
		}
	}

	rows, err := app.DB.QueryContext(ctx, `SELECT name, sentence, vote_count FROM answers WHERE question_name = $1 ORDER BY position`, questionName)
	if err != nil {
		// Query has an error
		log.Printf("Query error 2: %s", err.Error())
		w.WriteHeader(http.StatusInternalServerError)
		return
	}
	defer rows.Close()

	for rows.Next() {
		var answer TemplateAnswer

		err = rows.Scan(&answer.Name, &answer.Sentence, &answer.VoteCount)
		if err != nil {
			log.Printf("Query error 3: %s", err.Error())
			w.WriteHeader(http.StatusInternalServerError)
			return
		}

		templateVars.Answers = append(templateVars.Answers, answer)
	}

	// Compute the total vote count
	for i := range templateVars.Answers {
		templateVars.TotalVoteCount += templateVars.Answers[i].VoteCount
	}

	if templateVars.VoteOpen || templateVars.TotalVoteCount == 0 {
		template = templateChoice
		shuffleAnswers(templateVars.Answers)
	} else {
		// If the vote is closed, and there is at least one recorded vote, then consider the vote done,
		// and switch to the 'score' template to show the results
		template = templateScore
	}

	err = template.Execute(w, templateVars)
	if err != nil {
		// Template has an error
		log.Printf("Template execution error: %s", err.Error())
		w.WriteHeader(http.StatusInternalServerError)
		return
	}
	return
}

// Pseudorandom shuffle to prevent order bias in answers
func shuffleAnswers(answers []TemplateAnswer) {
	rnd := rand.New(rand.NewSource(time.Now().UnixNano()))
	for i := len(answers) - 1; i > 0; i-- {
		j := rnd.Intn(i + 1)
		answers[i], answers[j] = answers[j], answers[i]
	}
}
